@@ -79,7 +79,7 @@ use constant {
 sub IPROTOCLASS () { 'MR::IProto' }
 
 use vars qw/$VERSION %ERRORS/;
-$VERSION = 0.0.12;
+$VERSION = 0.0.13;
 
 BEGIN { *confess = \&MR::IProto::confess }
 
@@ -194,9 +194,14 @@ Properly ordered arrayref of fields' numbers which are indexed.
 
 =item B<default_index> => $default_index_name_string_or_id_uint32
 
-Index C<id> or C<name> to be used by default for the current C<space>.
+Index C<id> or C<name> to be used by default for the current C<space> in B<select> operations.
 Must be set if there are more than one C<\%index>es.
 
+=item B<primary_key_index> => $primary_key_name_string_or_id_uint32
+
+Index C<id> or C<name> to be used by default for the current C<space> in B<update> operations.
+It is set to C<default_index> by default.
+
 =back
 
 =item B<default_space> => $default_space_name_string_or_id_uint32
@@ -277,6 +282,7 @@ sub new {
     $self->{select_timeout}  = $arg->{select_timeout} || $self->{timeout};
     $self->{iprotoclass}     = $arg->{iprotoclass} || $class->IPROTOCLASS;
     $self->{_last_error}     = 0;
+    $self->{_last_error_msg} = '';
 
     $self->{hashify}         = $arg->{'hashify'} if exists $arg->{'hashify'};
     $self->{default_raw}     = $arg->{default_raw};
@@ -319,8 +325,11 @@ sub new {
         if( @{$ns->{indexes}} > 1 ) {
             confess "space[$namespace] default_index not given" unless defined $ns->{default_index};
             confess "space[$namespace] default_index $ns->{default_index} does not exist" unless $inames->{$ns->{default_index}};
+            $ns->{primary_key_index} = $ns->{default_index} unless defined $ns->{primary_key_index};
+            confess "space[$namespace] primary_key_index $ns->{primary_key_index} does not exist" unless $inames->{$ns->{primary_key_index}};
         } else {
             $ns->{default_index} ||= 0;
+            $ns->{primary_key_index} ||= 0;
         }
         if($ns->{fields}) {
             confess "space[$namespace] fields must be ARRAYREF" unless ref $ns->{fields} eq 'ARRAY';
@@ -405,48 +414,81 @@ sub _chat {
     my $soft_retry = $self->{softretry};
     my $retry_count = 0;
 
-    while ($retry > 0) {
+    my $callback  = delete $param{callback};
+    my $return_fh = delete $param{return_fh};
+    my $_cb = $callback || $return_fh;
+
+    my $is_retry = sub {
+        my ($data) = @_;
         $retry_count++;
-
-        $self->{_last_error} = 0x77777777;
-        $self->{server}->SetTimeout($timeout);
-        my $ret = $self->{server}->Chat1(%param);
-        my $message;
-
-        if (exists $ret->{ok}) {
-            my ($ret_code, $data, $full_code) = @{$ret->{ok}};
-            $self->{_last_error} = $full_code;
-            if ($ret_code->[0] == 0) {
-                my $ret = $orig_unpack->($$data,$ret_code->[2]);
-                confess __LINE__."$self->{name}: [common]: Bad response (more data left)" if length $$data > 0;
-                return $ret;
-            }
-
-            $self->{_last_error_msg} = $message = $ret_code->[0] == 0 ? "ok" : sprintf "Error %08X: %s", $full_code, $$data || $ERRORS{$full_code & 0xFFFFFF00} || 'Unknown error';
-            $self->_debug("$self->{name}: $message") if $self->{debug} >= 1;
-            if ($ret_code->[0] == 2) { #fatal error
-                $self->_raise($message) if $self->{raise};
-                return 0;
-            }
-
+        if($data) {
+            my ($ret_code, $data, $full_code) = @$data;
+            return 0 if $ret_code->[0] == 0;
             # retry if error is soft even in case of update e.g. ROW_LOCK
             if ($ret_code->[0] == 1 and --$soft_retry > 0) {
                 --$retry if $retry > 1;
-                sleep $self->{retry_delay};
-                next;
+                return 1;
+            }
+        }
+        return 1 if --$retry;
+        return 0;
+    };
+
+    my $message;
+    my $process = sub {
+        my ($data, $error) = @_;
+        my $errno = $!;
+        if (!$error && $data) {
+            my ($ret_code, $data, $full_code) = @$data;
+
+            $self->{_last_error} = $full_code;
+            $self->{_last_error_msg} = $message = $ret_code->[0] == 0 ? "ok" : sprintf "Error %08X: %s", $full_code, $$data || $ERRORS{$full_code & 0xFFFFFF00} || 'Unknown error';
+            $self->_debug("$self->{name}: $message") if $ret_code->[0] != 0 && $self->{debug} >= 1;
+
+            if ($ret_code->[0] == 0) {
+                my $ret = $orig_unpack->($$data,$ret_code->[2]);
+                confess __LINE__."$self->{name}: [common]: Bad response (more data left)" if length $$data > 0;
+                return $ret unless $_cb;
+                return &$_cb($ret);
+            }
+
+            if ($ret_code->[0] == 2) { #fatal error
+                $self->_raise($message) if $self->{raise};
+                return 0 unless $_cb;
+                return &$_cb(0, $error);
             }
         } else { # timeout has caused the failure if $ret->{timeout}
             $self->{_last_error} = 'fail';
-            $message ||= $self->{_last_error_msg} = $ret->{fail};
+            $message ||= $self->{_last_error_msg} = $error;
             $self->_debug("$self->{name}: $message") if $self->{debug} >= 1;
+            $self->_raise("$self->{name}: no success after $retry_count tries: $message\n") if $self->{raise};
+            return 0 unless $_cb;
+            return &$_cb(0, $error);
         }
+    };
 
-        last unless --$retry;
+    if ($callback) {
+        $self->{_last_error} = 0x77777777;
+        $self->{server}->SetTimeout($timeout);
+        return 1 if eval { $self->{server}->send({%param, is_retry => $is_retry}, $process); 1 };
+        return 0;
+    }
 
+    $param{continue} = $process if $return_fh;
+
+    my $ret;
+    while ($retry > 0) {
+        $self->{_last_error} = 0x77777777;
+        $self->{server}->SetTimeout($timeout);
+
+        $ret = $self->{server}->Chat1(%param);
+        return $ret->{ok} if $param{continue} && $ret->{ok};
+        last unless &$is_retry($ret->{ok});
         sleep $self->{retry_delay};
     };
 
-    $self->_raise("no success after $retry_count tries\n") if $self->{raise};
+    $self->_raise("no success after $retry_count tries\n") if $self->{raise} && !$ret->{ok};
+    return &$process($ret->{ok}, $ret->{fail});
 }
 
 sub _raise {
@@ -461,6 +503,7 @@ sub _validate_param {
     my %pnames = map { $_ => 1 } @pnames;
     $pnames{space} = 1;
     $pnames{namespace} = 1;
+    $pnames{callback} = 1;
     foreach my $pname (keys %$param) {
         confess "$self->{name}: unknown param $pname\n" unless exists $pnames{$pname};
     }
@@ -470,7 +513,7 @@ sub _validate_param {
     confess "$self->{name}: bad space `$param->{namespace}'" unless exists $self->{namespaces}->{$param->{namespace}};
 
     my $ns = $self->{namespaces}->{$param->{namespace}};
-    $param->{use_index} = $ns->{default_index} unless defined $param->{use_index};
+    $param->{use_index} = $pnames{use_index} ? $ns->{default_index} : $ns->{primary_key_index} unless defined $param->{use_index};
     confess "$self->{name}: bad index `$param->{use_index}'" unless exists $ns->{index_names}->{$param->{use_index}};
     $param->{index} = $ns->{index_names}->{$param->{use_index}};
 
@@ -572,11 +615,11 @@ The difference between them is the behaviour concerning tuple with the same prim
 
 =item *
 
-B<Add> will succeed if and only if duplicate-key tuple B<does not exist> 
+B<Add> will succeed if and only if duplicate-key tuple B<does not exist>
 
 =item *
 
-B<Replace> will succeed if and only if a duplicate-key tuple B<exists> 
+B<Replace> will succeed if and only if a duplicate-key tuple B<exists>
 
 =item *
 
@@ -642,17 +685,27 @@ sub Insert {
 
     $self->_debug("$self->{name}: INSERT[${\join '   ', map {join' ',unpack'(H2)*',$_} @tuple}]") if $self->{debug} >= 4;
 
+    my $cb = sub {
+        my ($r) = @_;
+
+        if($param->{want_result}) {
+            $self->_PostSelect($r, $param, $namespace);
+            $r = $r && $r->[0];
+        }
+
+        return $param->{callback}->($r) if $param->{callback};
+        return $r;
+    };
+
     my $r = $self->_chat (
         msg      => 13,
         payload  => pack("LLL (w/a*)*", $namespace->{namespace}, $flags, scalar(@tuple), @tuple),
         unpack   => sub { $self->_unpack_affected($flags, $namespace, @_) },
-        callback => $param->{callback},
+        callback => $param->{callback} && $cb,
     ) or return;
 
-    return $r unless $param->{want_result};
-
-    $self->_PostSelect($r, $param, $namespace);
-    return $r->[0];
+    return 1 if $param->{callback};
+    return $cb->($r);
 }
 
 sub _unpack_select {
@@ -749,7 +802,7 @@ sub _PackSelect {
             @$_{qw/field offset length/}
         } @{$param->{format}};
     }
-    return pack("LLLL a* La*", $namespace->{namespace}, $param->{index}->{id}, $param->{offset} || 0, $param->{limit} || scalar(@keys), $format, scalar(@keys), join('',@keys));
+    return pack("LLLL a* La*", $namespace->{namespace}, $param->{index}->{id}, $param->{offset} || 0, $param->{limit} || ($param->{default_limit_by_keys} ? scalar(@keys) : 0x7FFFFFFF), $format, scalar(@keys), join('',@keys));
 }
 
 sub _PostSelect {
@@ -773,13 +826,13 @@ Select tuple(s) from storage
     my $key = $id;
     my $key = [ $firstname, $lastname ];
     my @keys = ($key, ...);
-    
+
     my $tuple  = $box->Select($key)              or $box->Error && die $box->ErrorStr;
     my $tuple  = $box->Select($key, \%options)   or $box->Error && die $box->ErrorStr;
-    
+
     my @tuples = $box->Select(@keys)             or $box->Error && die $box->ErrorStr;
     my @tuples = $box->Select(@keys, \%options)  or $box->Error && die $box->ErrorStr;
-    
+
     my $tuples = $box->Select(\@keys)            or die $box->ErrorStr;
     my $tuples = $box->Select(\@keys, \%options) or die $box->ErrorStr;
 
@@ -836,9 +889,7 @@ Specify index (by id or name) to use.
 
 =item B<limit> => $limit_uint32
 
-Max tuples to select. It is set to C<< scalar @keys >> by default.
-To select all tuples (for example, using non-unique index), you
-should supply a big number here.
+Max tuples to select. It is set to C<< MAX_INT32 >> by default.
 
 =item B<raw> => $bool
 
@@ -857,7 +908,7 @@ C<False> will be returned in case of error.
 
 =cut
 
-my @select_param_ok = qw/use_index raw want next_rows limit offset raise hashify timeout format hash_by/;
+my @select_param_ok = qw/use_index raw want next_rows limit offset raise hashify timeout format hash_by callback return_fh default_limit_by_keys/;
 sub Select {
     confess q/Select isnt callable in void context/ unless defined wantarray;
     my ($param, $namespace) = $_[0]->_validate_param(\@_, @select_param_ok);
@@ -881,6 +932,47 @@ sub Select {
     local $namespace->{unpack_format} = $param->{unpack_format} if $param->{unpack_format};
 
     my $r = [];
+
+    $param->{want} ||= !1;
+    my $wantarray = wantarray;
+
+    my $cb = sub {
+        my ($r) = (@_);
+
+        $self->_PostSelect($r, $param, $namespace);
+
+        if ($r && defined(my $p = $param->{hash_by})) {
+            my %h;
+            if (@$r) {
+                if (ref $r->[0] eq 'HASH') {
+                    confess "Bad hash_by `$p' for HASH" unless exists $r->[0]->{$p};
+                    $h{$_->{$p}} = $_ for @$r;
+                } elsif (ref $r->[0] eq 'ARRAY') {
+                    confess "Bad hash_by `$p' for ARRAY" unless $p =~ m/^\d+$/ && $p >= 0 && $p < @{$r->[0]};
+                    $h{$_->[$p]} = $_ for @$r;
+                } else {
+                    confess "i dont know how to hash_by ".ref($r->[0]);
+                }
+            }
+            $r = \%h;
+        }
+
+        if ($param->{callback}) {
+            return $param->{callback}->($r);
+        }
+
+        return $r if $param->{hash_by};
+        return $r if $param->{want} eq 'arrayref';
+        $wantarray = wantarray if $param->{return_fh};
+        
+        if ($wantarray) {
+            return @{$r};
+        } else {
+            confess "$self->{name}: too many keys in scalar context" if @keys > 1;
+            return $r->[0];
+        }
+    };
+
     if (@keys && $payload) {
         $r = $self->_chat(
             msg      => $msg,
@@ -888,38 +980,16 @@ sub Select {
             unpack   => sub { $self->_unpack_select($namespace, "SELECT", @_) },
             retry    => $self->{select_retry},
             timeout  => $param->{timeout} || $self->{select_timeout},
-            callback => $param->{callback},
+            callback => $param->{callback} ? $cb : 0,
+            return_fh=> $param->{return_fh} ? $cb : 0,
         ) or return;
-    }
-
-    $param->{want} ||= !1;
-
-    $self->_PostSelect($r, $param, $namespace);
-
-    if(defined(my $p = $param->{hash_by})) {
-        my %h;
-        if(@$r) {
-            if (ref $r->[0] eq 'HASH') {
-                confess "Bad hash_by `$p' for HASH" unless exists $r->[0]->{$p};
-                $h{$_->{$p}} = $_ for @$r;
-            } elsif(ref $r->[0] eq 'ARRAY') {
-                confess "Bad hash_by `$p' for ARRAY" unless $p =~ m/^\d+$/ && $p >= 0 && $p < @{$r->[0]};
-                $h{$_->[$p]} = $_ for @$r;
-            } else {
-                confess "i dont know how to hash_by ".ref($r->[0]);
-            }
-        }
-        return \%h;
-    }
-
-    return $r if $param->{want} eq 'arrayref';
-
-    if (wantarray) {
-        return @{$r};
+        return $r if $param->{return_fh};
+        return 1 if $param->{callback};
     } else {
-        confess "$self->{name}: too many keys in scalar context" if @keys > 1;
-        return $r->[0];
+        $r = [];
     }
+
+    return $cb->($r);
 }
 
 sub SelectUnion {
@@ -963,7 +1033,7 @@ Delete tuple from storage. Return false upon error.
     my $n_deleted = $box->Delete($key) or die $box->ErrorStr;
     my $n_deleted = $box->Delete($key, \%options) or die $box->ErrorStr;
     warn "Nothing was deleted" unless int $n_deleted;
-    
+
     my $deleted_tuple_set = $box->Delete($key, { want_deleted_tuples => 1 }) or die $box->ErrorStr;
     warn "Nothing was deleted" unless @$deleted_tuple_set;
 
@@ -1001,17 +1071,27 @@ sub Delete {
     confess "$self->{name}\->Delete: for now key cardinality of 1 is only allowed" unless 1 == @{$param->{index}->{keys}};
     $self->_pack_keys($namespace, $param->{index}, $key);
 
+    my $cb = sub {
+        my ($r) = @_;
+
+        if($param->{want_result}) {
+            $self->_PostSelect($r, $param, $namespace);
+            $r = $r && $r->[0];
+        }
+
+        return $param->{callback}->($r) if $param->{callback};
+        return $r;
+    };
+
     my $r = $self->_chat(
         msg      => $flags ? 21 : 20,
         payload  => $flags ? pack("L L a*", $namespace->{namespace}, $flags, $key) : pack("L a*", $namespace->{namespace}, $key),
         unpack   => sub { $self->_unpack_affected($flags, $namespace, @_) },
-        callback => $param->{callback},
+        callback => $param->{callback} && $cb,
     ) or return;
 
-    return $r unless $param->{want_result};
-
-    $self->_PostSelect($r, $param, $namespace);
-    return $r->[0];
+    return 1 if $param->{callback};
+    return $cb->($r);
 }
 
 sub OP_SET          () { 0 }
@@ -1075,11 +1155,11 @@ BEGIN {
 Apply several update operations to a tuple.
 
     my @op = ([ f1 => add => 10 ], [ f1 => and => 0xFF], [ f2 => set => time() ], [ misc_string => cutend => 3 ]);
-    
+
     my $n_updated = $box->UpdateMulti($key, @op) or die $box->ErrorStr;
     my $n_updated = $box->UpdateMulti($key, @op, \%options) or die $box->ErrorStr;
     warn "Nothing was updated" unless int $n_updated;
-    
+
     my $updated_tuple_set = $box->UpdateMulti($key, @op, { want_result => 1 }) or die $box->ErrorStr;
     warn "Nothing was updated" unless @$updated_tuple_set;
 
@@ -1124,7 +1204,7 @@ Append or prepend C<< $field >> with C<$value> string.
 
 Cut C<< $value >> bytes from beginning or end of C<< $field >>.
 
-=back 
+=back
 
 =back
 
@@ -1148,7 +1228,7 @@ sub UpdateMulti {
     my ($param, $namespace) = $_[0]->_validate_param(\@_, qw/want_updated_tuple want_result _flags raw/);
     my ($self, $key, @op) = @_;
 
-    $self->_debug("$self->{name}: UPDATEMULTI(NS:$namespace->{namespace},KEY:$key)[@{[map{qq{[@$_]}}@op]}]") if $self->{debug} >= 3;
+    $self->_debug("$self->{name}: UPDATEMULTI(NS:$namespace->{namespace},KEY:$key)[@{[map{$_?qq{[@$_]}:q{-}}@op]}]") if $self->{debug} >= 3;
 
     confess "$self->{name}\->UpdateMulti: for now key cardinality of 1 is only allowed" unless 1 == @{$param->{index}->{keys}};
     confess "$self->{name}: too many op" if scalar @op > 128;
@@ -1203,17 +1283,27 @@ sub UpdateMulti {
 
     $self->_pack_keys($namespace, $param->{index}, $key);
 
+    my $cb = sub {
+        my ($r) = @_;
+
+        if($param->{want_result}) {
+            $self->_PostSelect($r, $param, $namespace);
+            $r = $r && $r->[0];
+        }
+
+        return $param->{callback}->($r) if $param->{callback};
+        return $r;
+    };
+
     my $r = $self->_chat(
         msg      => 19,
         payload  => pack("LL a* L (a*)*" , $namespace->{namespace}, $flags, $key, scalar(@op), @op),
         unpack   => sub { $self->_unpack_affected($flags, $namespace, @_) },
-        callback => $param->{callback},
+        callback => $param->{callback} && $cb,
     ) or return;
-
-    return $r unless $param->{want_result};
-
-    $self->_PostSelect($r, $param, $namespace);
-    return $r->[0];
+    
+    return 1 if $param->{callback};
+    return $cb->($r);
 }
 
 sub Update {
@@ -1262,6 +1352,51 @@ sub Num {
     $self->UpdateMulti($key, @op, $param);
 }
 
+=head2 AnyEvent
+
+C<< Insert, UpdateMulti, Select, Delete, Call >> methods can be given the following options:
+
+=over
+
+=item B<callback> => sub { my ($data, $error) = @_; }
+
+Async request using AnyEvent.
+C<< $data >> is unpacked and processed according to request options data.
+C<< $error >> contains a message string in case of error.
+
+=back
+
+=head2 "Continuations"
+
+C<< Select >> methods can be given the following options:
+
+=over
+
+=item <return_fh> => 1
+
+The request does only send operation on network, and returns
+C<< { fh => $IO::Handle, continue => $code } >>. C<< $code >> reads data from network,
+unpacks, processes according to options and returns it.
+
+You should handle timeouts manually (using select() call for example).
+Usage example:
+
+    my $continuation = $box->Select(13,{ return_fh => 1 });
+    ok $continuation, "select/continuation";
+    
+    my $rin = '';
+    vec($rin,$continuation->{fh}->fileno,1) = 1;
+    my $ein = $rin;
+    ok 0 <= select($rin,undef,$ein,2), "select/continuation/select";
+    
+    my $res = $continuation->{continue}->();
+    use Data::Dumper;
+    is_deeply $res, [13, 'some_email@test.mail.ru', 1, 2, 3, 4, '123456789'], "select/continuation/result";
+
+=back
+
+
+
 =head1 LICENCE AND COPYRIGHT
 
 This is free software; you can redistribute it and/or modify it under the same terms as the Perl 5 programming language system itself.
